/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2000-2001 CodeFactory AB
 * Copyright (C) 2000-2001 Richard Hult <rhult@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <bonobo.h>
#include <bonobo/bonobo-listener.h>
#include <bonobo/bonobo-property-bag.h>
#include <liboaf/liboaf.h>
#include "util/type-utils.h"
#include "util/corba-utils.h"
#include "client/widgets/mr-message-box.h"
#include "client.h"

#define PROJECT_OAFIID "OAFIID:GNOME_MrProject_Project"
#define PARENT_TYPE BONOBO_X_OBJECT_TYPE

#define d(x) 

static GtkObjectClass *parent_class;

/* GtkObject. */
static void shell_init       (Shell *shell);
static void shell_class_init (ShellClass *klass);

/* Private members. */
struct _ShellPriv {
	GNOME_MrProject_Project  project_co;
	BonoboUIContainer       *container;
	BonoboEventSource       *event_source;
	BonoboListener          *listener;
	BonoboPropertyBag       *property_bag;
};

/* Signals. */ 
enum {
	NOT_USED,
	LAST_SIGNAL
};

static GNOME_MrProject_Project
impl_GNOME_MrProjectShell_getProject (PortableServer_Servant  servant,
				      CORBA_Environment      *ev)
{
	Shell                   *shell;
	ShellPriv               *priv;
	GNOME_MrProject_Project  project_co;

	shell = SHELL (bonobo_object_from_servant (servant));
	priv = shell->priv;

	if (priv->project_co == CORBA_OBJECT_NIL) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_Shell_HasNoProject,
				     NULL);

		return CORBA_OBJECT_NIL;
	}

	project_co = CORBA_Object_duplicate (priv->project_co, ev);
	return project_co;
}


static Bonobo_UIContainer
impl_GNOME_MrProjectShell_getUIContainer (PortableServer_Servant servant, CORBA_Environment *ev)
{
	Shell                   *shell;
	shell = SHELL (bonobo_object_from_servant (servant));
	
	return bonobo_object_dup_ref (bonobo_object_corba_objref (BONOBO_OBJECT (shell->priv->container)), NULL);
}

static void
shell_destroy (GtkObject *object)
{
	Shell                 *shell;
	ShellPriv             *priv;
	GNOME_MrProject_Shell  shell_co;
	CORBA_Environment      ev;

	puts (__FUNCTION__);
	
	shell = SHELL (object);
	priv = shell->priv;

	CORBA_exception_init (&ev);
	if (shell->priv->project_co != CORBA_OBJECT_NIL) {
		shell_co = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
		GNOME_MrProject_Project_removeShell (priv->project_co, shell_co, &ev);

		bonobo_object_release_unref (priv->project_co, &ev);
		if (BONOBO_EX (&ev)) 
			g_log_exception (&ev, "");

		priv->project_co = CORBA_OBJECT_NIL;
	}

	CORBA_exception_free (&ev);

	g_free (shell->priv);
	shell->priv = NULL;

	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
shell_class_init (ShellClass *klass)
{
	POA_GNOME_MrProject_Shell__epv *epv = &klass->epv;
	GtkObjectClass *object_class = (GtkObjectClass *) klass;

	object_class->destroy = shell_destroy;

	epv->getProject = impl_GNOME_MrProjectShell_getProject;
	epv->getUIContainer = impl_GNOME_MrProjectShell_getUIContainer;
}

static void shell_init (Shell *shell)
{
	shell->priv = g_new0 (ShellPriv, 1);
}

static void
listener_callback (BonoboListener    *listener,
		   char              *event_name, 
		   CORBA_any         *any,
		   CORBA_Environment *ev,
		   gpointer           user_data)
{
	Shell *shell;

	shell = (Shell *) user_data;

	bonobo_event_source_notify_listeners (shell->priv->event_source,
					      event_name,
					      any,
					      ev);
}

static void
set_prop (BonoboPropertyBag *bag,
	  const BonoboArg   *arg,
	  guint              arg_id,
	  CORBA_Environment *ev,
	  gpointer           user_data)
{
}

static void
get_prop (BonoboPropertyBag *bag,
	  BonoboArg         *arg,
	  guint              arg_id,
	  CORBA_Environment *ev,
	  gpointer           user_data)
{
}

static Shell *
shell_construct (Shell *shell, BonoboUIContainer *container)
{
	BonoboEventSource *event_source;
	BonoboListener    *listener;
	BonoboPropertyBag *pb;

	g_return_val_if_fail (shell != NULL, NULL);
	g_return_val_if_fail (IS_SHELL (shell), NULL);

	shell->priv->container = container;
	
	pb = bonobo_property_bag_new (get_prop, set_prop, shell);
	bonobo_object_add_interface (BONOBO_OBJECT (shell), BONOBO_OBJECT (pb));
	/* Add shell settings etc here later...
	   bonobo_property_bag_add (pb, "prop", 1, BONOBO_ARG_STRING, NULL, "descr", 0);
	*/
	shell->priv->property_bag = pb;
	
	event_source = bonobo_event_source_new ();
	bonobo_object_add_interface (BONOBO_OBJECT (shell), BONOBO_OBJECT (event_source));
	shell->priv->event_source = event_source;

	listener = bonobo_listener_new (listener_callback, shell);
	bonobo_object_add_interface (BONOBO_OBJECT (shell), BONOBO_OBJECT (listener));
	shell->priv->listener = listener;
	
	return shell;
}

Shell *
shell_new (BonoboUIContainer *container)
{
	Shell *shell;
	
	shell = gtk_type_new (shell_get_type ());
	if (!shell_construct (shell, container))
		return NULL;

	return shell;
}

void
shell_open (Shell *shell, const gchar *uri, CORBA_Environment *ev)
{
	ShellPriv             *priv;
	GNOME_MrProject_Shell  shell_co;
	CORBA_any	      *any;
	
	g_return_if_fail (shell != NULL);
	g_return_if_fail (IS_SHELL (shell));

	d(puts (__FUNCTION__));

	priv = shell->priv;
	shell_co = bonobo_object_corba_objref (BONOBO_OBJECT (shell));

	/* a project only present, destroy it */
	if (priv->project_co) {
		g_print ("Removing old project\n");
		
		any = CORBA_any__alloc ();
		any->_type = TC_GNOME_MrProject_Project;
		any->_value = priv->project_co;
		CORBA_any_set_release (any, FALSE);
		
		bonobo_event_source_notify_listeners (priv->event_source,
						      "GNOME/MrProject:project:unset",
						      any,
						      NULL);
		CORBA_free (any);
		GNOME_MrProject_Project_removeShell (priv->project_co, shell_co, ev);

		Bonobo_Unknown_unref(priv->project_co, ev);
		/* FIXME: check for errors */
		priv->project_co = CORBA_OBJECT_NIL;
	} 

	priv->project_co = oaf_activate_from_id (PROJECT_OAFIID,
						 0,    /* OAF_ActivationFlags */
						 NULL, /* OAF_ActivationID */
						 ev);

	if (BONOBO_EX (ev) || priv->project_co == CORBA_OBJECT_NIL) {
		g_warning ("Could not activate Project.");
		return;
	}
	
	GNOME_MrProject_Project_addShell (priv->project_co,
					  shell_co,
					  ev);
	
	any = CORBA_any__alloc ();
	any->_type = TC_GNOME_MrProject_Project;
	any->_value = priv->project_co;
	CORBA_any_set_release (any, FALSE);
	
	bonobo_event_source_notify_listeners (priv->event_source,
					      "GNOME/MrProject:project:set",
					      any,
					      NULL);
	
	CORBA_free (any);
	

	if (uri != NULL && strlen (uri) > 0) {
		GNOME_MrProject_Project_load (priv->project_co,
					      uri,
					      ev);
	}
}

void
shell_save (Shell *shell, const gchar *uri, CORBA_Environment *ev)
{
	GNOME_MrProject_Shell shell_co;

	g_return_if_fail (shell != NULL);
	g_return_if_fail (IS_SHELL (shell));
	g_return_if_fail (uri != NULL);

	/* Save the project data and the state of the shell and
	 * its components.
	 */

	/* Plan:
	* 1. Tell the project to save itself.
	* 2. Pass a stream for the shell to add to the storage, plus a path.
	* 3. Pass streams+paths for all the components.
	*/
	
	shell_co = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
	GNOME_MrProject_Project_save (shell->priv->project_co, uri, ev);
	if (BONOBO_EX (ev)) {
		return;
	}

	/* FIXME: Don't hardcode the path here, should be the name used for the shell. */
	/*GNOME_MrProject_Project_saveSubdirectory (shell->priv->project_co, shell_co, "Shell", ev);*/
	if (BONOBO_EX (ev)) {
		return;
	}

	/* FIXME: save components here. */
}

GNOME_MrProject_Project
shell_get_project (Shell *shell)
{
	g_return_val_if_fail (shell != NULL, CORBA_OBJECT_NIL);
	g_return_val_if_fail (IS_SHELL (shell), CORBA_OBJECT_NIL);

	return shell->priv->project_co;
}

void
shell_set_range (Shell *shell, time_t first, time_t last)
{
	BonoboArg         *arg;
	CORBA_Environment  ev;

	g_return_if_fail (shell != NULL);
	g_return_if_fail (IS_SHELL (shell));

	CORBA_exception_init (&ev);
	
	arg = bonobo_arg_new (BONOBO_ARG_INT);

	BONOBO_ARG_SET_INT (arg, first);
	bonobo_event_source_notify_listeners (shell->priv->event_source,
					      "Bonobo/Property:change:FirstTime",
					      (CORBA_any *) arg,
					      &ev);

	BONOBO_ARG_SET_INT (arg, last);
	bonobo_event_source_notify_listeners (shell->priv->event_source,
					      "Bonobo/Property:change:LastTime",
					      (CORBA_any *) arg,
					      &ev);

	CORBA_exception_free (&ev);
	bonobo_arg_release (arg);
}


BONOBO_X_TYPE_FUNC_FULL (Shell,
			 GNOME_MrProject_Shell,
			 PARENT_TYPE,
			 shell);
