/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2000-2001 CodeFactory AB
 * Copyright (C) 2000-2001 Richard Hult <rhult@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <bonobo.h>
#include <bonobo/bonobo-listener.h>
#include <liboaf/liboaf.h>
#include "util/type-utils.h"
#include "client-component.h"

/* GtkObject. */
static void  shell_component_init         (ShellComponent          *component);
static void  shell_component_class_init   (ShellComponentClass     *klass);
static void  shell_component_set_shell    (ShellComponent          *component,
					   GNOME_MrProject_Shell    shell, 
					   CORBA_Environment       *ev);
static void  shell_component_unset_shell  (ShellComponent          *component,
					   CORBA_Environment       *ev);

static Bonobo_Control shell_component_create_view (ShellComponent   *component,
						   CORBA_Environment *ev);


#define d(x)
#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
static GtkObjectClass *parent_class;

static void
impl_GNOME_MrProject_ShellComponent_setShell (PortableServer_Servant  servant, 
					      GNOME_MrProject_Shell   shell, 
					      CORBA_Environment      *ev)
{
	ShellComponent        *component;
	ShellComponentClass   *klass;
	
	d(puts (__FUNCTION__));
		
	component = SHELL_COMPONENT (bonobo_object_from_servant (servant));
	
	klass = SHELL_COMPONENT_CLASS (GTK_OBJECT (component)->klass);
	klass->set_shell (component, shell, ev);
}

static void
impl_GNOME_MrProject_ShellComponent_unsetShell (PortableServer_Servant servant, 
						CORBA_Environment      *ev)
{
	ShellComponent        *component;
	ShellComponentClass   *klass;

	d(puts (__FUNCTION__));
	
	component = SHELL_COMPONENT (bonobo_object_from_servant (servant));

	klass = SHELL_COMPONENT_CLASS (GTK_OBJECT (component)->klass);
	klass->unset_shell (component, ev);
}

static Bonobo_Control
impl_GNOME_MrProject_ShellComponent_createView (PortableServer_Servant servant, 
						CORBA_Environment *ev)
{
	ShellComponent        *component;
	ShellComponentClass   *klass;

	d(puts (__FUNCTION__));
	
	component = SHELL_COMPONENT (bonobo_object_from_servant (servant));

	klass = SHELL_COMPONENT_CLASS (GTK_OBJECT (component)->klass);
	return klass->create_view (component, ev);
}

static void
shell_component_destroy (GtkObject *object)
{
	ShellComponent      *shell_component;
	CORBA_Environment    ev;

	shell_component = SHELL_COMPONENT (object);

	CORBA_exception_init (&ev);

	if (shell_component->shell != CORBA_OBJECT_NIL) {
		CORBA_Object_release (shell_component->shell, &ev);
		shell_component->shell = CORBA_OBJECT_NIL;
	}
	CORBA_exception_free (&ev);

	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
shell_component_class_init (ShellComponentClass *klass)
{
	POA_GNOME_MrProject_ShellComponent__epv   *epv;
	GtkObjectClass                            *object_class;


	epv          = &klass->epv;
	object_class = (GtkObjectClass *) klass;
	parent_class = gtk_type_class (PARENT_TYPE);

	object_class->destroy = shell_component_destroy;

	klass->set_shell = shell_component_set_shell;
	klass->unset_shell = shell_component_unset_shell;
	klass->create_view = shell_component_create_view;

	epv->setShell = impl_GNOME_MrProject_ShellComponent_setShell;
	epv->unsetShell = impl_GNOME_MrProject_ShellComponent_unsetShell;
	epv->createView = impl_GNOME_MrProject_ShellComponent_createView;
}

static void 
shell_component_init (ShellComponent *component)
{
	component->shell = CORBA_OBJECT_NIL;
}

static void
shell_component_set_shell (ShellComponent          *component, 
			   GNOME_MrProject_Shell    shell, 
			   CORBA_Environment       *ev)
{
	d(puts (__FUNCTION__));
	
	if (component->shell != CORBA_OBJECT_NIL) {
		/* FIXME: What to do? */
		g_warning ("Can't set shell, already got one.");
		return;
	}

	component->shell = CORBA_Object_duplicate (shell, ev);
}

static void
shell_component_unset_shell (ShellComponent *component, CORBA_Environment *ev)
{
	if (component->shell == CORBA_OBJECT_NIL) {
		/* FIXME: raise exception. */
		return;
	}

	CORBA_Object_release (component->shell, ev);
	component->shell = CORBA_OBJECT_NIL;
}

static Bonobo_Control
shell_component_create_view (ShellComponent *component, CORBA_Environment *ev)
{
	/* Raise exception. */
	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
			     ex_GNOME_MrProject_ShellComponent_UnsupportedMethod,
			     NULL);
	
	return CORBA_OBJECT_NIL;
}

BONOBO_X_TYPE_FUNC_FULL (ShellComponent, 
			 GNOME_MrProject_ShellComponent,
			 PARENT_TYPE,
			 shell_component);
