/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2001 CodeFactory AB
 * Copyright (C) 2001 Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Mikael Hallendal <micke@codefactory.se>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h> /* must be included before gnome-dateedit.h> */
#include <gal/widgets/e-unicode.h>
#include <gal/e-table/e-table.h>
#include <gal/e-table/e-table-extras.h>
#include <gal/e-table/e-table-scrolled.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkentry.h>
#include <gtk/gtklabel.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtksignal.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/gnome-dateedit.h>
#include <glade/glade.h>
#include <stdio.h> /* Remove when puts go away */
#include "task-dialog-gui.h"
#include "alloc-table-model.h"
#include "util/corba-utils.h"
#include "util/type-utils.h"

static void      task_dialog_gui_init          (TaskDialogGui    *gui);
static void      task_dialog_gui_class_init    (GtkObjectClass   *klass);

static void      tdg_destroy                   (GtkObject        *object);

static gboolean  tdg_name_changed_cb           (GtkWidget        *object,
						GdkEventFocus    *focus_event,
						gpointer          user_data);
static void      tdg_date_time_changed_cb      (GnomeDateEdit    *date_edit,
						gpointer          user_data);
static void      tdg_complete_changed_cb       (GtkWidget        *complete_sb,
						gpointer          user_data);
static void      tdg_priority_changed_cb       (GtkWidget        *priority_sb,
						gpointer          user_data);

static void tdg_atm_resource_allocated_cb   (AllocTableModel       *atm,
					     GNOME_MrProject_Id     res_id,
					     gpointer               user_data);
static void tdg_atm_resource_deallocated_cb (AllocTableModel       *atm,
					     GNOME_MrProject_Id     res_id,
					     gpointer               user_data);

static void      tdg_create_gui                (TaskDialogGui    *gui);
static void      tdg_connect_signals           (TaskDialogGui    *gui);

#define d(x)

struct _TaskDialogGuiPriv {
	GM_Task       *task;
	
	GtkWidget     *notebook;

	/* Page one, properties */
        GtkWidget     *label_id;
        GtkWidget     *entry_name;
        GtkWidget     *date_start;
        GtkWidget     *date_end;
        GtkWidget     *sb_complete;
        GtkWidget     *sb_priority;
	GtkWidget     *button_close;

	/* Page two, allocations */
	GtkWidget     *alloc_table;
	ETableModel   *atm;
};

enum {
        TASK_UPDATED,
        DEPENDENCY_ADDED,
        DEPENDENCY_REMOVED,
        RESOURCE_ALLOCATED,
        RESOURCE_DEALLOCATED,
        LAST_SIGNAL
};

static gint signals[LAST_SIGNAL] = { 0 };

GNOME_CLASS_BOILERPLATE (TaskDialogGui,  task_dialog_gui,
                         GtkWindow,      gtk_window);


static void
task_dialog_gui_init (TaskDialogGui *gui)
{
        TaskDialogGuiPriv   *priv;
        
        priv       = g_new0 (TaskDialogGuiPriv, 1);
        gui->priv  = priv;

	gtk_window_set_wmclass (GTK_WINDOW (gui),
				"Task Properties",
				"MrProject");
	gtk_window_set_title (GTK_WINDOW (gui),
			      _("Task Properties - MrProject"));
	
        tdg_create_gui (gui);
        tdg_connect_signals (gui);
}

static void
task_dialog_gui_class_init (GtkObjectClass *klass)
{
        g_return_if_fail (klass != NULL);
        g_return_if_fail (IS_TASK_DIALOG_GUI_CLASS (klass));
        
        /* GtkObject */
        klass->destroy = tdg_destroy;

        /* Signals */
        signals[TASK_UPDATED] = 
                gtk_signal_new ("task_updated",
                                GTK_RUN_LAST,
                                klass->type,
                                GTK_SIGNAL_OFFSET (TaskDialogGuiClass,
                                                   task_updated),
                                gtk_marshal_NONE__POINTER,
                                GTK_TYPE_NONE,
                                1, GTK_TYPE_POINTER);

        signals[DEPENDENCY_ADDED] =
                gtk_signal_new ("dependency_added",
                                GTK_RUN_LAST,
                                klass->type,
                                GTK_SIGNAL_OFFSET (TaskDialogGuiClass,
                                                   dependency_added),
                                gtk_marshal_NONE__INT_INT,
                                GTK_TYPE_NONE,
                                2, GTK_TYPE_INT, GTK_TYPE_INT);

	signals[DEPENDENCY_REMOVED] =
		gtk_signal_new ("dependency_removed",
				GTK_RUN_LAST,
				klass->type,
				GTK_SIGNAL_OFFSET (TaskDialogGuiClass,
						   dependency_removed),
				gtk_marshal_NONE__INT,
				GTK_TYPE_NONE,
				1, GTK_TYPE_INT);

	signals[RESOURCE_ALLOCATED] =
		gtk_signal_new ("resource_allocated",
				GTK_RUN_LAST,
				klass->type,
				GTK_SIGNAL_OFFSET (TaskDialogGuiClass,
						   resource_allocated),
				gtk_marshal_NONE__INT_INT,
				GTK_TYPE_NONE,
				2, GTK_TYPE_INT, GTK_TYPE_INT);

	signals[RESOURCE_DEALLOCATED] = 
		gtk_signal_new ("resource_deallocated",
				GTK_RUN_LAST,
				klass->type,
				GTK_SIGNAL_OFFSET (TaskDialogGuiClass,
						   resource_deallocated),
				gtk_marshal_NONE__INT_INT,
				GTK_TYPE_NONE,
				2, GTK_TYPE_INT, GTK_TYPE_INT);
	
	gtk_object_class_add_signals (klass, signals, LAST_SIGNAL);
}

static void
tdg_destroy (GtkObject *object)
{
	TaskDialogGui   *gui;
	
	g_return_if_fail (object != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (object));

	gui = TASK_DIALOG_GUI (object);

	d(puts (__FUNCTION__));
}

static gboolean
tdg_name_changed_cb (GtkWidget       *widget,
		     GdkEventFocus   *focus_event,
		     gpointer         user_data)
{
	TaskDialogGui          *gui;
	GNOME_MrProject_Task   *task;
	gchar                  *text;
	
	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (GTK_IS_ENTRY (widget), FALSE);
	g_return_val_if_fail (user_data != NULL, FALSE);
	g_return_val_if_fail (IS_TASK_DIALOG_GUI (user_data), FALSE);
	
	gui   = TASK_DIALOG_GUI (user_data);
	task  = gui->priv->task;

	text = e_utf8_gtk_entry_get_text (GTK_ENTRY (widget));
	
	if (strcmp (task->name, text)) {
		CORBA_free (task->name);
		task->name = CORBA_string_dup (text);
		gtk_signal_emit (GTK_OBJECT (gui), signals[TASK_UPDATED], task);
	}
	
	g_free (text);

	return FALSE;
}

static void
tdg_date_time_changed_cb (GnomeDateEdit   *date_edit,
			  gpointer         user_data)
{
	TaskDialogGui          *gui;
	TaskDialogGuiPriv      *priv;
	GNOME_MrProject_Task   *task;
	time_t                  time;
	gboolean                changed = TRUE;
	
	g_return_if_fail (date_edit != NULL);
	g_return_if_fail (GNOME_IS_DATE_EDIT (date_edit));
	g_return_if_fail (user_data != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (user_data));
	
	gui   = TASK_DIALOG_GUI (user_data);
	priv  = gui->priv;
	task  = priv->task;

	time = gnome_date_edit_get_date (date_edit);
	
	if (date_edit == GNOME_DATE_EDIT(priv->date_start)) {
		if (task->start != time) {
			changed = TRUE;
			task->start = time;
		}
	}
	else if (date_edit == GNOME_DATE_EDIT (gui->priv->date_end)) {
		if (task->end != time) {
			changed = TRUE;
			task->end = time;
		}
	}

	if (changed) {
		gtk_signal_emit (GTK_OBJECT (gui), signals[TASK_UPDATED], task);
	}
}

static void
tdg_complete_changed_cb (GtkWidget   *complete_sb,
			 gpointer     user_data)
{
	d(puts (__FUNCTION__));
}

static void
tdg_priority_changed_cb (GtkWidget   *priority_sb,
			 gpointer     data)
{
	d(puts (__FUNCTION__));
}

static gint
tdg_key_event (GtkWidget   *widget,
	       GdkEventKey *event,
	       GtkWidget   *close_button)
{
	if (event->keyval == GDK_Escape) {
		gtk_button_clicked (GTK_BUTTON (close_button));
		return TRUE;
	} else {
		return FALSE;
	}
}

static void
tdg_atm_resource_allocated_cb   (AllocTableModel       *atm,
				 GNOME_MrProject_Id     res_id,
				 gpointer               user_data)
{
	TaskDialogGui   *gui;
	
	g_return_if_fail (user_data != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (user_data));
	
	gui = TASK_DIALOG_GUI (user_data);

	gtk_signal_emit (GTK_OBJECT (gui), 
			 signals[RESOURCE_ALLOCATED],
			 gui->priv->task->taskId,
			 res_id);
}

static void
tdg_atm_resource_deallocated_cb (AllocTableModel       *atm,
				 GNOME_MrProject_Id     res_id,
				 gpointer               user_data)
{
	TaskDialogGui   *gui;
	
	g_return_if_fail (user_data != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (user_data));
	
	gui = TASK_DIALOG_GUI (user_data);

	gtk_signal_emit (GTK_OBJECT (gui), 
			 signals[RESOURCE_DEALLOCATED],
			 gui->priv->task->taskId,
			 res_id);
}

static void
tdg_create_gui (TaskDialogGui *gui) 
{
        GladeXML            *glade;
        TaskDialogGuiPriv   *priv;
        GtkWidget           *window;
	GtkWidget           *contents;
	GtkWidget           *label;
	
        priv = gui->priv;

        glade = glade_xml_new (MRPROJECT_DATADIR "task-dialog-gui.glade",
                               "task_dialog");
	
	window              = glade_xml_get_widget (glade, "task_dialog");
	
	contents            = glade_xml_get_widget (glade, "dialog_contents");

	gtk_widget_reparent (contents, GTK_WIDGET (gui));
	gtk_widget_destroy  (window);

	priv->notebook      = glade_xml_get_widget (glade, "notebook");
	
        priv->label_id      = glade_xml_get_widget (glade, "label_id");
        priv->entry_name    = glade_xml_get_widget (glade, "entry_name");
	priv->date_start    = glade_xml_get_widget (glade, "date_start");
        priv->date_end      = glade_xml_get_widget (glade, "date_end");
        priv->sb_complete   = glade_xml_get_widget (glade, "sb_complete");
        priv->sb_priority   = glade_xml_get_widget (glade, "sb_priority");
	priv->button_close  = glade_xml_get_widget (glade, "button_close");

	gtk_signal_connect (GTK_OBJECT (gui), "key_press_event",
			    GTK_SIGNAL_FUNC (tdg_key_event), priv->button_close);
	
	priv->atm = alloc_table_model_new ();
	
	priv->alloc_table = e_table_scrolled_new_from_spec_file (
		E_TABLE_MODEL (priv->atm),
		e_table_extras_new (),
		MRPROJECT_DATADIR "task-dialog-alloc-table.etspec",
		MRPROJECT_DATADIR "task-dialog-alloc-table.etstate");

	gtk_widget_show (priv->alloc_table);
	label = gtk_label_new (_("Resources"));
	gtk_widget_show (label);
	
	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook),
				  priv->alloc_table,
				  label);
	
	gtk_object_unref (GTK_OBJECT (glade));
}

static void
tdg_connect_signals (TaskDialogGui *gui)
{
        TaskDialogGuiPriv   *priv;
        
        priv = gui->priv;

	gtk_signal_connect (GTK_OBJECT (priv->entry_name),
			    "focus_out_event",
			    GTK_SIGNAL_FUNC (tdg_name_changed_cb),
			    GTK_OBJECT (gui));

	gtk_signal_connect (GTK_OBJECT (priv->date_start),
			    "date_changed",
			    GTK_SIGNAL_FUNC (tdg_date_time_changed_cb),
			    GTK_OBJECT (gui));
	
	gtk_signal_connect (GTK_OBJECT (priv->date_start),
			    "time_changed",
			    GTK_SIGNAL_FUNC (tdg_date_time_changed_cb),
			    GTK_OBJECT (gui));

	gtk_signal_connect (GTK_OBJECT (priv->date_end),
			    "date_changed",
			    GTK_SIGNAL_FUNC (tdg_date_time_changed_cb),
			    GTK_OBJECT (gui));
	
	gtk_signal_connect (GTK_OBJECT (priv->date_end),
			    "time_changed",
			    GTK_SIGNAL_FUNC (tdg_date_time_changed_cb),
			    GTK_OBJECT (gui));
			    
	gtk_signal_connect (GTK_OBJECT (priv->sb_complete),
			    "changed",
			    GTK_SIGNAL_FUNC (tdg_complete_changed_cb),
			    GTK_OBJECT (gui));
	
	gtk_signal_connect (GTK_OBJECT (priv->sb_priority),
			    "changed",
			    GTK_SIGNAL_FUNC (tdg_priority_changed_cb),
			    GTK_OBJECT (gui));

	gtk_signal_connect (GTK_OBJECT (priv->atm),
			    "resource_allocated",
			    GTK_SIGNAL_FUNC (tdg_atm_resource_allocated_cb),
			    GTK_OBJECT (gui));

	gtk_signal_connect (GTK_OBJECT (priv->atm),
			    "resource_deallocated",
			    GTK_SIGNAL_FUNC (tdg_atm_resource_deallocated_cb),
			    GTK_OBJECT (gui));
	
	gtk_signal_connect_object (GTK_OBJECT (priv->button_close),
				   "clicked",
				   GTK_SIGNAL_FUNC (gtk_object_destroy),
				   GTK_OBJECT (gui));
}

static void
tdg_update_gui (TaskDialogGui   *gui)
{
	TaskDialogGuiPriv      *priv;
	GNOME_MrProject_Task   *task;
	gchar                  *id;
	
	priv  = gui->priv;
	task  = priv->task;

	if (!task) {
		g_warning ("Trying to update GUI without a task");
		return;
	}

	id = g_strdup_printf ("%d", task->taskId);

	gtk_label_set_text (GTK_LABEL (priv->label_id), id);
	e_utf8_gtk_entry_set_text (GTK_ENTRY (priv->entry_name), priv->task->name);
	gnome_date_edit_set_time (GNOME_DATE_EDIT (priv->date_start), 
				  task->start);
	gnome_date_edit_set_time (GNOME_DATE_EDIT (priv->date_end), task->end);

	g_free (id);
}

GtkWidget  *
task_dialog_gui_new (void)
{
	TaskDialogGui   *gui;
	
	gui = gtk_type_new (TASK_DIALOG_GUI_TYPE);
	
	return GTK_WIDGET (gui);
}

void  
task_dialog_gui_update_task (TaskDialogGui          *gui,
			     GNOME_MrProject_Task   *task)
{
	TaskDialogGuiPriv   *priv;
	
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));
	g_return_if_fail (task != NULL);
	
	priv = gui->priv;

	if (!priv->task) {
		priv->task = corba_util_task_duplicate (task);
		tdg_update_gui (gui);
		return;
	}

	if (priv->task->taskId != task->taskId) {
		return;
	}

	if (!corba_util_task_update (gui->priv->task, task,
					  TASK_CHANGE_ALL)) {
		return;
	}

	tdg_update_gui (gui);
}

void  
task_dialog_gui_add_resource (TaskDialogGui              *gui,
			      GNOME_MrProject_Resource   *res)
{
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));
	g_return_if_fail (res != NULL);

	alloc_table_model_add_resource (ALLOC_TABLE_MODEL (gui->priv->atm),
					res);
}

void 
task_dialog_gui_update_resource (TaskDialogGui              *gui,
				 GNOME_MrProject_Resource   *res)
{
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));
	g_return_if_fail (res != NULL);

	alloc_table_model_update_resource (ALLOC_TABLE_MODEL (gui->priv->atm), 
					   res);
}

void  
task_dialog_gui_remove_resource (TaskDialogGui        *gui,
				 GNOME_MrProject_Id    id)
{
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));

	alloc_table_model_remove_resource (ALLOC_TABLE_MODEL (gui->priv->atm),
					   id);
}

void
task_dialog_gui_set_allocation (TaskDialogGui        *gui,
				GNOME_MrProject_Id    id,
				gboolean              alloc)
{
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));

	alloc_table_model_set_allocation (ALLOC_TABLE_MODEL (gui->priv->atm),
					  id, alloc);
}

void
task_dialog_gui_add_group (TaskDialogGui                   *gui,
			   GNOME_MrProject_ResourceGroup   *group)
{
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));
	g_return_if_fail (group != NULL);

	d(puts (__FUNCTION__));
}

void  
task_dialog_gui_remove_group (TaskDialogGui        *gui,
			      GNOME_MrProject_Id    gid)
{
	g_return_if_fail (gui != NULL);
	g_return_if_fail (IS_TASK_DIALOG_GUI (gui));

	d(puts (__FUNCTION__));
}








