/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2000-2001 CodeFactory AB
 * Copyright (C) 2000-2001 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2001      Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_EVOLUTION_COMPOSER_IDL
#include "libmrproject/Composer.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <gdk/gdkkeysyms.h>
#include <bonobo/bonobo-ui-util.h>
#include <bonobo/bonobo-win.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <libgnomevfs/gnome-vfs-result.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#include <liboaf/liboaf.h>
#include <liboaf/oaf-async.h>
#include <gal/shortcut-bar/e-shortcut-bar.h>
#include <gal/e-paned/e-hpaned.h>
#include <glade/glade.h>
#include "util/type-utils.h"
#include "util/corba-utils.h"
#include "util/eel-vfs-extensions.h"
#include "client/widgets/mr-hpaned.h"
#include "client/widgets/mr-message-box.h"
#include "project-window.h"
#include "project-properties-dialog.h"
#include "template-dialog.h"
#include "client.h"

#define COMPOSER_IID "OAFIID:GNOME_Evolution_Mail_Composer"

static void     project_window_init        (GtkObject      *object);
static void     project_window_class_init  (GtkObjectClass *object_class);
static gchar   *window_title_from_name     (const gchar    *str);
static gchar   *get_project_name           (ProjectWindow  *window);
static void     get_project_range          (ProjectWindow  *window);
static void     add_shell_listener         (ProjectWindow  *window);
static gboolean project_window_delete      (ProjectWindow  *window, 
					    gpointer        user_data);
static void	project_window_exit        (ProjectWindow  *window);
static gboolean project_window_really_exit (ProjectWindow  *window);
static void     project_window_add_control (ProjectWindow  *window,
					    Bonobo_Control  control_co,
					    GtkWidget      *widget,
					    GdkPixbuf      *pixbuf,
					    const gchar    *title);

GNOME_CLASS_BOILERPLATE (ProjectWindow, project_window, 
			 BonoboWindow, bonobo_window);


static GList *window_list = NULL;
extern GSList *open_projects;

struct _ProjectWindowPriv {
	BonoboUIContainer *container;
	BonoboUIComponent *component;
	Shell             *shell;
	BonoboListener    *listener;
	gint               listener_id;
	GtkWidget         *notebook;
	GList             *controls;
	Bonobo_Control     active_control;
	GtkWidget         *shortcut_bar;
	GtkWidget         *paned;
};

static void
project_window_destroy (GtkObject *object)
{
	ProjectWindow             *window;
	Bonobo_Control          control_co;
	GNOME_MrProject_ShellComponent component_co;
	GList                  *list;
	CORBA_Environment       ev;
	GNOME_MrProject_Shell   shell_co;
	GNOME_MrProject_Project project_co;

	window = PROJECT_WINDOW (object);
	
	CORBA_exception_init (&ev);
	
	for (list = window->priv->controls; list; list = list->next) {
		control_co = list->data;

		component_co = Bonobo_Unknown_queryInterface (
			control_co,
			"IDL:GNOME/MrProject/ShellComponent:1.0",
			&ev);

		/* Does not export component interface. */
		if (BONOBO_EX (&ev) || component_co == CORBA_OBJECT_NIL) {
			CORBA_exception_free (&ev);
			CORBA_exception_init (&ev);
		} else {
			GNOME_MrProject_ShellComponent_unsetShell (component_co, &ev);
			if (BONOBO_EX (&ev)) {
				g_warning ("Could not unset shell.");
			}
			bonobo_object_release_unref (component_co, NULL);
		}
	}

	bonobo_object_unref (BONOBO_OBJECT (window->priv->container));

	/* We need to do this slightly hackish thing, because the event source
	   in Project holds a ref to the Shell (since the Listener is aggregated),
	   so we remove the shell here.
	*/
	shell_co = BONOBO_OBJREF (window->priv->shell);
	project_co = GNOME_MrProject_Shell_getProject (shell_co, &ev);
	if (BONOBO_EX (&ev))
		g_warning ("Could not get project (%s).", 
			   bonobo_exception_get_text (&ev));
	else {
		GNOME_MrProject_Project_removeShell (project_co, shell_co, &ev);
		if (BONOBO_EX (&ev))
			g_warning ("Could not remove shell (%s).",
				   bonobo_exception_get_text (&ev));
	}
	
	CORBA_exception_free (&ev);

	bonobo_object_unref (BONOBO_OBJECT (window->priv->shell));

	g_free (window->priv);
	window->priv = NULL;

	open_projects = g_slist_remove (open_projects, project_co);

	window_list = g_list_remove (window_list, window);
	if (g_list_length (window_list) == 0) {
		gtk_main_quit ();
	}
}

static void
new_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	GtkWidget *window;
	window = project_window_new (NULL);
	gtk_widget_show_all (window);
}

static void
new_details_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	GtkWidget *dialog, *window;
	gint       ret;
	gchar     *filename;
	
	dialog = template_dialog_new ();
	ret =  template_dialog_run (TEMPLATE_DIALOG (dialog), &filename);

	g_print ("Filename is: %s\n", filename);
	gtk_widget_destroy (dialog);

	if (ret != GNOME_OK) {
		return;
	}

	if (filename) {
		GM_Project         project_co;
		CORBA_Environment  ev;
		gchar             *uri;

		/* FIXME: use gnomevfs uri code */
		uri = g_strconcat ("file://", filename, NULL);
		
		window = project_window_new (uri);
	
		/* Reset the filename so we don't try to save over it. */
		project_co = shell_get_project (PROJECT_WINDOW (window)->priv->shell);
		CORBA_exception_init (&ev);

		GNOME_MrProject_Project_setURI (project_co, "", &ev);
		CORBA_exception_free (&ev);

		g_free (uri);
	}
	else {
		window = project_window_new (NULL);
	}
		
	gtk_widget_show_all (window);
	g_free (filename);
}

static void
file_sel_handle_cancel (GtkWidget *widget, gpointer user_data)
{
	GtkWidget *fsel;

	fsel = gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION);
	gtk_widget_hide (fsel);
	gtk_main_quit ();
}

static void
file_sel_handle_ok (GtkWidget *widget, gboolean *dialog_result)
{
	struct stat       sb;
	GtkFileSelection *file_sel;
	char             *name;

	file_sel = GTK_FILE_SELECTION (
		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
	name = gtk_file_selection_get_filename (file_sel);

	/* Change into directory if that's what user selected */
	if ((stat (name, &sb) == 0) && S_ISDIR (sb.st_mode)) {
		char *last_slash = strrchr (name, '/');
		gchar *dirname;
		/* The file selector needs a '/' at the end of a
		   directory name */
		if (!last_slash || *(last_slash + 1) != '\0')
			dirname = g_strconcat (name, "/", NULL);
		else
			dirname = g_strdup (name);
		gtk_file_selection_set_filename (file_sel, dirname);
		g_free (dirname);
	} else {
		*dialog_result = TRUE;
		gtk_main_quit ();
	}
}

static guint
file_sel_delete_event (GtkWidget *widget, GdkEventAny *event)
{
	gtk_widget_hide (widget);
	gtk_main_quit ();
	return TRUE;
}

static gboolean 
file_sel_key_event (GtkFileSelection *file_sel, GdkEventKey *event)
{
	if (event->keyval == GDK_Escape) {
		gtk_button_clicked (GTK_BUTTON (file_sel->cancel_button));
		return TRUE;
	} else
		return FALSE;
}

static void
open_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow    *window, *new_window;
	GtkFileSelection *file_sel;
	gboolean          accepted;
	gchar            *uri;

	window = PROJECT_WINDOW (data);

	file_sel =  GTK_FILE_SELECTION (gtk_file_selection_new (
		_("Open Project")));
	gtk_window_set_wmclass (GTK_WINDOW (file_sel),
				"FileSelector",
				"MrProject");
	/*gtk_file_selection_complete (file_sel, "*.mrproject");*/ 
	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
	
	/* Connect the signals for Ok and Cancel */
	accepted = FALSE;
	gtk_signal_connect (GTK_OBJECT (file_sel->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (file_sel_handle_ok), &accepted);
	gtk_signal_connect (GTK_OBJECT (file_sel->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (file_sel_handle_cancel), NULL);
	gtk_signal_connect (GTK_OBJECT (file_sel), "key_press_event",
			    GTK_SIGNAL_FUNC (file_sel_key_event), NULL);

	gtk_window_set_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);

	gtk_signal_connect (GTK_OBJECT (file_sel), "delete_event",
			    GTK_SIGNAL_FUNC (file_sel_delete_event), NULL);

	gtk_widget_show (GTK_WIDGET (file_sel));
	gtk_grab_add (GTK_WIDGET (file_sel));
	gtk_main ();

	if (accepted) {
		GNOME_MrProject_Project       project_co;
		CORBA_Environment             ev;
		GNOME_MrProject_ProjectState  state;
		gchar                        *name;

		uri = g_strconcat ("file://",
				   gtk_file_selection_get_filename (file_sel),
				   NULL);
		gtk_widget_destroy (GTK_WIDGET (file_sel));

		project_co = shell_get_project (window->priv->shell);
		CORBA_exception_init (&ev);

		state = GNOME_MrProject_Project_getState (project_co, &ev);
		if (BONOBO_EX (&ev)) {
			g_log_exception (&ev, "");

			CORBA_exception_free (&ev);
			g_free (uri);
			return;
		}

		if (state == GNOME_MrProject_PROJECT_STATE_EMPTY) {
			/* The project we already have is empty,
			 * use that to load the new project into.
			 */
			GNOME_MrProject_Project_load (project_co, uri, &ev);

			if (BONOBO_EX (&ev)) {
				g_log_exception (&ev, "");

				CORBA_exception_free (&ev);
				g_free (uri);
				return;
			}

			open_projects = g_slist_prepend (open_projects, project_co);
		} else {
			/* Ok, we already have a project that is being used.
			 * Open a new window for our new project.
			 */
			new_window = PROJECT_WINDOW (project_window_new (uri));
			gtk_widget_show_all (GTK_WIDGET (new_window));
			
			name = get_project_name (new_window);
			gtk_window_set_title (GTK_WINDOW (new_window), name);
		}
		
		CORBA_exception_free (&ev);
		g_free (uri);
	} else {
		gtk_widget_destroy (GTK_WIDGET (file_sel));
	}
}

static void
save_as_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow *window;
	GtkFileSelection *file_sel;
	gboolean accepted;
	gchar *uri;

	window = (ProjectWindow *) data;
	
	file_sel =  GTK_FILE_SELECTION (gtk_file_selection_new (
		_("Save Project as")));
	gtk_window_set_wmclass (GTK_WINDOW (file_sel),
				"FileSelector",
				"MrProject");
	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);

	/* Connect the signals for Ok and Cancel */
	accepted = FALSE;
	gtk_signal_connect (GTK_OBJECT (file_sel->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (file_sel_handle_ok), &accepted);
	gtk_signal_connect (GTK_OBJECT (file_sel->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (file_sel_handle_cancel), NULL);
	gtk_signal_connect (GTK_OBJECT (file_sel), "key_press_event",
			    GTK_SIGNAL_FUNC (file_sel_key_event), NULL);

	gtk_window_set_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);

	gtk_signal_connect (GTK_OBJECT (file_sel), "delete_event",
			    GTK_SIGNAL_FUNC (file_sel_delete_event), NULL);

	gtk_widget_show (GTK_WIDGET (file_sel));
	gtk_grab_add (GTK_WIDGET (file_sel));
	gtk_main ();

	if (accepted) {
		GNOME_MrProject_Project  project_co;
		CORBA_Environment        ev;
		gchar                   *name;

		project_co = shell_get_project (window->priv->shell);

		name = gtk_file_selection_get_filename (file_sel);
		if (strstr (name, ".mrproject") == NULL) {
			uri = g_strconcat ("file://", name, ".mrproject", NULL);
		} else {
			uri = g_strconcat ("file://", name, NULL);
		}			
		
		CORBA_exception_init (&ev);
		GNOME_MrProject_Project_save (project_co, uri, &ev);
		if (BONOBO_EX (&ev)) {
			g_log_exception (&ev, "");
		}

		g_free (uri);
		CORBA_exception_free (&ev);
	}
				      
	gtk_widget_destroy (GTK_WIDGET (file_sel));
}

static void
save_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow *window;
	GNOME_MrProject_Project project_co;
	CORBA_Environment ev;
	CORBA_char *uri;

	window = (ProjectWindow *) data;
	project_co = shell_get_project (window->priv->shell);
	
	CORBA_exception_init (&ev);

	uri = GNOME_MrProject_Project_getURI (project_co, &ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not get URI (%s).", 
			   bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);
		return;
	}

	if (uri == CORBA_OBJECT_NIL || strlen (uri) == 0) {
		/* Dispatch to save as. */
		save_as_cb (component, data, cname); /* FIXME: Ugly. */
	} else {
		GNOME_MrProject_Project_save (project_co, "", &ev);
		if (BONOBO_EX (&ev)) {
			g_warning ("Could not save (%s).",
				   bonobo_exception_get_text (&ev));
		}
	}
	
	CORBA_exception_free (&ev);
}

static void
print_setup_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
}

static void
print_preview_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
}

static void
revert_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow *window;
	GNOME_MrProject_Project project_co;
	CORBA_char *uri;
	CORBA_Environment ev;

	window = PROJECT_WINDOW (data);
	
	CORBA_exception_init (&ev);

	project_co = shell_get_project (window->priv->shell);
	g_print("Attempting to revert file\n");
	/* retrieve the uri of current project */
	uri = GNOME_MrProject_Project_getURI (project_co, &ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not get URI (%s).", 
			   bonobo_exception_get_text (&ev));

		goto revert_error;
	}

	/* make sure it is semi-valid and load it */
	if (uri != CORBA_OBJECT_NIL && strlen (uri) > 0) {
		g_print ("Reverting to %s\n", uri);
		shell_open(window->priv->shell, uri, &ev);

		if (BONOBO_EX (&ev)) {
			g_log_exception (&ev, "reverting file");
			goto revert_error;
		}
	}

	/* handle error and do all other kinds of stuff */
 revert_error:	
	CORBA_exception_free (&ev);
	if (uri) {
		CORBA_free(uri);
	}
}

static void
properties_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow         *window;
	GNOME_MrProject_Shell  shell_co;

	window = PROJECT_WINDOW (data);
	shell_co = BONOBO_OBJREF (window->priv->shell);
	
	project_properties_dialog_run (data, shell_co);
}

#ifdef HAVE_EVOLUTION_COMPOSER_IDL
static void
composer_component_loaded (CORBA_Object  composer,
			   const gchar  *error_reason,
			   gpointer      user_data)
{
	gchar                        *uri;
	gint                          file_size;
	gchar                        *file_contents;
	gchar                        *file_basename;
	GnomeVFSResult                result;
	GNOME_MrProject_Project       project_co;
	CORBA_Environment             ev;
	GNOME_Evolution_Composer_RecipientList *list;
	ProjectWindow                *window;

	window = (ProjectWindow *)user_data;

	project_co = shell_get_project (window->priv->shell);

	bonobo_ui_component_set_status (window->priv->component, "", NULL);

	if (composer == CORBA_OBJECT_NIL) {
		mr_message_box_show_message (GNOME_MESSAGE_BOX_ERROR,
					     "Unable to start mail composer component!");
		return;
	}

	CORBA_exception_init (&ev);
	
	/* Get the URI */
	uri = GNOME_MrProject_Project_getURI (project_co, &ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not get URI (%s).", 
			   bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);
		return;
	}

	result = eel_read_entire_file (uri, &file_size, &file_contents);
	file_basename = eel_uri_get_basename (uri);

	g_print ("Result after reading file is: %d\n", result);
	CORBA_free (uri);

	if (result != GNOME_VFS_OK) {
		g_warning ("There was an error reading the file (%s).",
			   gnome_vfs_result_to_string (result));
		return;
	}

	/* Set the subject */
	list = GNOME_Evolution_Composer_RecipientList__alloc ();
	list->_length = 0;
	list->_maximum = 0;
	list->_buffer = CORBA_sequence_GNOME_Evolution_Composer_Recipient_allocbuf (0);

	GNOME_Evolution_Composer_setHeaders (composer, list, list, list,
					     file_basename, &ev);

	if (BONOBO_EX (&ev)) {
		g_warning ("Could not attach data (%s).", 
			   bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);

		CORBA_free (list);
		
		g_free (file_contents);
		g_free (file_basename);

		return;
	}

	/* Attach the file */
	GNOME_Evolution_Composer_attachData (
		composer,
		gnome_vfs_mime_type_from_name (file_basename),
		file_basename,
		"",
		FALSE,
		file_contents,
		&ev);
	
	g_free (file_contents);
	g_free (file_basename);
	
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not attach data (%s).", 
			   bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);

		return;
	}

	/* Show the file */
	GNOME_Evolution_Composer_show (composer, &ev);
	CORBA_exception_free (&ev);
}
#endif

static void
send_project_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
#ifdef HAVE_EVOLUTION_COMPOSER_IDL
	ProjectWindow                *window;
	GNOME_MrProject_Project       project_co;
	GNOME_MrProject_ProjectState  state;
	CORBA_Environment             ev;

	window = (ProjectWindow *) data;
	project_co = shell_get_project (window->priv->shell);
	
	CORBA_exception_init (&ev);
	state = GNOME_MrProject_Project_getState (project_co, &ev);
	if (BONOBO_EX (&ev)) {
		g_log_exception (&ev, "");
		
		CORBA_exception_free (&ev);
		return;
	}
	
	if (state == GNOME_MrProject_PROJECT_STATE_EMPTY ||
	    state == GNOME_MrProject_PROJECT_STATE_DIRTY) {
		GtkWidget *dialog;
		gint       result;

		dialog = gnome_message_box_new (_("The project has not been saved,\n"
                                                  "you must save your project before sending it.\n"
						  "Do you wish to do so?"),
						GNOME_MESSAGE_BOX_QUESTION,
						GNOME_STOCK_BUTTON_YES,
						GNOME_STOCK_BUTTON_NO);

		gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
		result = gnome_dialog_run (GNOME_DIALOG (dialog));

		if (result == 0) {
			/* FIXME: A little ugly */
			save_cb (component, data, cname);
		}
	}

	state = GNOME_MrProject_Project_getState (project_co, &ev);

	/* Check if the user saved the project */
	if (state != GNOME_MrProject_PROJECT_STATE_SAVED) {
		return;
	}

	oaf_activate_from_id_async (COMPOSER_IID,
				    0,
				    composer_component_loaded,
				    window,
				    &ev);

	if (ev._major != CORBA_NO_EXCEPTION) {
		CORBA_exception_free (&ev);
		mr_message_box_show_message (GNOME_MESSAGE_BOX_ERROR,
					     "Unable to start mail composer component!");
		return;
	}

	bonobo_ui_component_set_status (window->priv->component,
					_("Loading composer ..."),
					NULL);
#endif
}

static void
close_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow                     *window;
	window = PROJECT_WINDOW (data);
	
	if (project_window_really_exit (window)) {
		gtk_object_destroy (GTK_OBJECT (window));
	}
}

static void
exit_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow *window;

	window = PROJECT_WINDOW (data);

	project_window_exit (window);
}

static void
undo_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
}

static void
redo_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
}

static void
preferences_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
}

static void
about_cb (BonoboUIComponent *component, gpointer data, const char *cname)
{
	ProjectWindow *window;
	GtkWidget     *about, *href;

	const gchar *authors[] = {
		"Richard Hult <rhult@codefactory.se>",
		"Mikael Hallendal <micke@codefactory.se>",
		NULL
 	};

 	window = PROJECT_WINDOW (data);
	
 	about = gnome_about_new (_("MrProject"), VERSION,
				 _("Copyright (C) 2000-2001 CodeFactory AB"),
 				 authors,
 				 _("A project management application for the GNOME Desktop Environment"),
				 MRPROJECT_IMAGEDIR "cflogo_bw_67x87.png");

	href = gnome_href_new ("http://mrproject.codefactory.se", _("The MrProject Homepage"));
	gtk_widget_show (href);

	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox), href, FALSE, FALSE, 0);
	
	gnome_dialog_set_parent (GNOME_DIALOG (about), GTK_WINDOW (window));
 	gnome_dialog_set_close (GNOME_DIALOG (about), TRUE);

 	gtk_widget_show (about);
}

static BonoboUIVerb verbs[] = {
	BONOBO_UI_VERB ("FileNew",		new_cb),
	BONOBO_UI_VERB ("FileNewDetails",	new_details_cb),
	BONOBO_UI_VERB ("FileOpen",		open_cb),
	BONOBO_UI_VERB ("FileSave",		save_cb),
	BONOBO_UI_VERB ("FileSaveAs",		save_as_cb),
	BONOBO_UI_VERB ("FileRevert",		revert_cb),
	BONOBO_UI_VERB ("FileProperties",	properties_cb),
	BONOBO_UI_VERB ("FilePrintSetup",	print_setup_cb),
	BONOBO_UI_VERB ("FilePrint",		print_preview_cb),
	BONOBO_UI_VERB ("FilePrintPreview",	print_preview_cb),
	BONOBO_UI_VERB ("FileSendProject",      send_project_cb),
	BONOBO_UI_VERB ("FileClose",		close_cb),
	BONOBO_UI_VERB ("FileExit",		exit_cb),

	BONOBO_UI_VERB ("EditUndo",		undo_cb),
	BONOBO_UI_VERB ("EditRedo",		redo_cb),

	BONOBO_UI_VERB ("SettingsPreferences",	preferences_cb),

	BONOBO_UI_VERB ("HelpAbout",		about_cb),

	BONOBO_UI_VERB_END
};

static void
project_window_init (GtkObject *object)
{
	ProjectWindow *window = PROJECT_WINDOW(object);
	window->priv = g_new0 (ProjectWindowPriv, 1);
}

static void
project_window_class_init (GtkObjectClass *object_class)
{
	object_class->destroy = project_window_destroy;
}

static void
project_window_shortcut_bar_item_selected (GtkWidget *widget, GdkEvent *event, gint item_num, ProjectWindow *window)
{
	gtk_notebook_set_page (GTK_NOTEBOOK (window->priv->notebook), item_num);
}

/* FIXME: Talk to gal maintainers about making this a public function? */
static void
e_shortcut_bar_set_canvas_style (GtkWidget *canvas)
{
        GtkRcStyle *rc_style;

        rc_style = gtk_rc_style_new ();

        rc_style->color_flags[GTK_STATE_NORMAL] = GTK_RC_FG | GTK_RC_BG;
        rc_style->fg[GTK_STATE_NORMAL].red   = 65535;
        rc_style->fg[GTK_STATE_NORMAL].green = 65535;
        rc_style->fg[GTK_STATE_NORMAL].blue  = 65535;

        rc_style->bg[GTK_STATE_NORMAL].red   = 32512;
        rc_style->bg[GTK_STATE_NORMAL].green = 32512;
        rc_style->bg[GTK_STATE_NORMAL].blue  = 32512;

        gtk_widget_modify_style (GTK_WIDGET (canvas), rc_style);
        gtk_rc_style_unref (rc_style);
}

static void
activate_control (ProjectWindow *window, Bonobo_Control control_co)
{
	CORBA_Environment ev;

	CORBA_exception_init (&ev);

	if (CORBA_Object_is_equivalent (control_co, window->priv->active_control, &ev)) {
		CORBA_exception_free (&ev);
		return;
	}
	
	Bonobo_Control_activate (control_co, TRUE, &ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not activate control.");
	} else {
		if (window->priv->active_control) {
			Bonobo_Control_activate (window->priv->active_control, FALSE, &ev);
		}
		window->priv->active_control = control_co;
	}

	CORBA_exception_free (&ev);
}

static void
notebook_switch_page_cb (GtkNotebook     *notebook,
			 GtkNotebookPage *page,
			 gint             page_num,
			 ProjectWindow      *window)
{
	GList *list;

	list = g_list_nth (window->priv->controls, page_num);
	if (list != NULL && list->data != NULL) {
		activate_control (window, list->data);
	}
}

/* Returns a list of languages, containing
   the LANG or LANGUAGE environment setting (with and without region code).
   The elements in the returned list must be freed */
static GSList *
get_lang_list (void)
{
        GSList *retval;
        char *lang, *lang_with_locale, *tmp, *org_pointer;
        char *equal_char;

        retval = NULL;

        tmp = g_getenv ("LANGUAGE");

        if (tmp == NULL) {
                tmp = g_getenv ("LANG");
        }

	lang = g_strdup (tmp);
	org_pointer = lang;

	if (lang != NULL) {
		/* envs can be in NAME=VALUE form */
		equal_char = strchr (lang, '=');
		if (equal_char != NULL) {
			lang = equal_char + 1;
		}
		
		/* lang may be in form LANG_LOCALE */
		equal_char = strchr (lang, '_');
		if (equal_char != NULL) {
			lang_with_locale = g_strdup (lang);
			*equal_char = 0;
		} else {
			lang_with_locale = NULL;
		}

		/* Make sure we don't give oaf an empty
		 * lang string.
		 */
		if (lang_with_locale != NULL && strlen (lang_with_locale) != 0) {
			retval = g_slist_prepend (retval, 
						  g_strdup (lang_with_locale));
		}
		g_free (lang_with_locale);
        }
	g_free (org_pointer);
        
        return retval;
}

static gboolean
does_the_evolution_composer_exist (void)
{
	OAF_ServerInfoList *oaf_result;
	CORBA_Environment ev;
	gchar *query;
	gboolean found = FALSE;
	
	query = "repo_ids.has ('IDL:GNOME/Evolution:Composer:1.0')";

	CORBA_exception_init (&ev);

	oaf_result = oaf_query (query, NULL, &ev);

	if (ev._major == CORBA_NO_EXCEPTION && oaf_result != NULL && oaf_result->_length > 0) {
		found = TRUE;
	}

	CORBA_free (oaf_result);
	CORBA_exception_free (&ev);

	return found;
}

/*
 * FIXME: Move this function to util since it seems to also be in
 *  gantt-component.c and probably somewhere in the resource component
 */
static void
set_pixmap (BonoboUIComponent *uic,
	    const char        *xml_path,
	    const char        *icon)
{
	char      *path;
	GdkPixbuf *pixbuf;

	path = g_concat_dir_and_file (MRPROJECT_IMAGEDIR, icon);

	pixbuf = gdk_pixbuf_new_from_file (path);
	if (pixbuf == NULL) {
		g_warning ("Cannot load image -- %s", path);
		g_free (path);
		return;
	}

	bonobo_ui_util_set_pixbuf (uic, xml_path, pixbuf);

	gdk_pixbuf_unref (pixbuf);

	g_free (path);
}

GtkWidget *
project_window_new (const gchar *uri)
{
	CORBA_Environment               ev;
	Bonobo_UIContainer              uic;
	ProjectWindow                     *window;
	ProjectWindowPriv                 *priv;
	GtkWidget                      *ret;
	int                             sx, sy;
	GNOME_MrProject_ShellComponent  component_co;
	Bonobo_Control                  control_co;
	
	window = gtk_type_new (PROJECT_TYPE_WINDOW);

	priv = window->priv;

	/* Create the Bonobo container. */
	priv->container = bonobo_ui_container_new ();
	bonobo_ui_container_set_win (priv->container, BONOBO_WINDOW (window));
	uic = bonobo_object_corba_objref (BONOBO_OBJECT (priv->container));

	priv->shell = shell_new (priv->container);

	/* Create the main window. */
	ret = bonobo_window_construct (BONOBO_WINDOW (window), "MrProject", "MrProject");
	gtk_signal_connect (GTK_OBJECT (ret), "delete_event", GTK_SIGNAL_FUNC (project_window_delete), NULL);
	gtk_window_set_policy (GTK_WINDOW (ret), TRUE, TRUE, FALSE);

	bonobo_ui_engine_config_set_path (
		bonobo_window_get_ui_engine (BONOBO_WINDOW (ret)),
		"/MrProject/UIConf/kvps");

	
	/* Set the default size of the window window. */
	sx = MAX (gdk_screen_width () - 64, 600);
	sy = MAX (gdk_screen_height () - 64, 480);
	sx = MIN (sx, 1024);
	sy = MIN (sy, 768);
	sx = (sx * 3) / 4;
	sy = (sy * 3) / 4;
	gtk_window_set_default_size (GTK_WINDOW (ret), sx, sy);

	window->priv->component = bonobo_ui_component_new ("MrProject");
	bonobo_ui_component_set_container (window->priv->component, uic);
	bonobo_ui_component_add_verb_list_with_data (window->priv->component,
						     verbs,
						     window);

	bonobo_ui_util_set_ui (window->priv->component, GNOME_DATADIR,
			       "GNOME_MrProject_Client.ui", "mrproject");

	bonobo_ui_component_freeze (window->priv->component, NULL);

	/* Hide the Send To submenu if we can't find the evolution composer */
	if (does_the_evolution_composer_exist () == FALSE) {
		bonobo_ui_component_set_prop (window->priv->component,
					      "/menu/File/Send",
					      "hidden", "1",
					      NULL);
		bonobo_ui_component_set_prop (window->priv->component,
					      "/menu/File/SendSep",
					      "hidden", "1",
					      NULL);
	}
	else {
		/* Set the send to pixmap */
		set_pixmap (window->priv->component,
			    "/menu/File/Send/FileSendProject",
			    "16_send_to.png");
	}

	priv->paned = mr_hpaned_new ();
	
	priv->notebook = gtk_notebook_new ();
	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (priv->notebook), FALSE);
	gtk_signal_connect (GTK_OBJECT (priv->notebook),
			    "switch-page",
			    notebook_switch_page_cb,
			    window);

	priv->shortcut_bar = e_icon_bar_new ();
	gtk_signal_connect (GTK_OBJECT (priv->shortcut_bar),
			    "item_selected",
			    GTK_SIGNAL_FUNC (project_window_shortcut_bar_item_selected),
			    window);
	
	e_shortcut_bar_set_canvas_style (priv->shortcut_bar);
	gtk_widget_set_usize (priv->shortcut_bar, 92, -1);
	
	e_paned_add1 (E_PANED (priv->paned), priv->shortcut_bar);
	e_paned_add2 (E_PANED (priv->paned), priv->notebook);
	e_paned_set_position (E_PANED (priv->paned), 90);
	
	bonobo_window_set_contents (BONOBO_WINDOW (ret), priv->paned);

	CORBA_exception_init (&ev);
	
	/* Testing: Try to activate some components. */
	{
		CORBA_Environment      ev;
		char                  *sort[2];
		const gchar           *query;
		OAF_ServerInfoList    *oaf_result;
		guint                  i;
		GSList                *langs;
		GtkWidget	      *bonobo_widget;
		GNOME_MrProject_Shell  shell_co;
		const gchar           *title, *icon_file;
		gchar                 *icon_file_full;
		OAF_ServerInfo         server;
		
		CORBA_exception_init (&ev);
		
		query = "mrproject:label.defined() AND "
			"mrproject:icon.defined() AND "
			"repo_ids.has ('IDL:GNOME/MrProject/ShellComponent:1.0')";

		/* Hack to get gantt chart first. Should look into some
		 * priority scheme or some such.
		 */
		sort[0] = "mrproject:supported_views.has ('gantt_chart')";
		sort[1] = NULL;
		
		oaf_result = oaf_query (query, sort, &ev);
		
		if (!BONOBO_EX (&ev) && oaf_result != NULL) {
			for (i = 0; i < oaf_result->_length; i++) {
				server = oaf_result->_buffer[i];

				component_co = oaf_activate_from_id (server.iid,
								     0,	   /* OAF_ActivationFlags */
								     NULL, /* OAF_ActivationID* */
								     &ev);
				
				if (BONOBO_EX (&ev) || !component_co) {
					gchar *msg = g_strdup_printf (
						"Could not activate the component: %s\n"
						"Make sure you followed the instructions in the README file.", 
						server.iid);
					g_message (msg);
					g_free (msg);

					CORBA_exception_free (&ev);
					continue;
				}
				
				control_co = GNOME_MrProject_ShellComponent_createView (
					component_co, &ev);
				
				if (BONOBO_EX (&ev) || !control_co) {
					g_message ("Could not get Bonobo::Control interface for component '%s'.",
						   server.iid);
					CORBA_exception_free (&ev);
				} else {
					shell_co = bonobo_object_corba_objref (BONOBO_OBJECT (priv->shell));
					GNOME_MrProject_ShellComponent_setShell (component_co, shell_co, &ev);
					if (BONOBO_EX (&ev)) {
						g_warning ("Could not set shell on new component.");
						CORBA_exception_free (&ev);
					}
				}

				bonobo_widget = bonobo_widget_new_control_from_objref (control_co, uic);

				langs = get_lang_list ();
				title = oaf_server_info_prop_lookup (&server, "mrproject:label", langs);
				if (!title) {
					title = _("Unknown");
				}

				icon_file = oaf_server_info_prop_lookup (&server, "mrproject:icon", NULL);
				if (!icon_file) {
					icon_file = "unknown.png";
				}
				
				icon_file_full = g_concat_dir_and_file (MRPROJECT_IMAGEDIR, icon_file);

				if (!g_file_exists (icon_file_full)) {
					g_free (icon_file_full);
					icon_file_full = NULL;
				}

				/* FIXME: The icon file should be transported over CORBA,
				 * but this will have to do for now.
				 */
				project_window_add_control (window,
							 control_co,
							 bonobo_widget,
							 icon_file_full ? gdk_pixbuf_new_from_file (icon_file_full) : NULL,
							 title);
				
				activate_control (window, control_co);
				g_free (icon_file_full);
			}
		}
		
		if (oaf_result) {
			CORBA_free (oaf_result);
		}
		
		CORBA_exception_free (&ev);
	}

	shell_open (priv->shell, uri, &ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not open project.");
	}

	{
		GNOME_MrProject_Project project_co;
		project_co = shell_get_project (priv->shell);
		open_projects = g_slist_prepend (open_projects, project_co);
	}

	/* FIXME: eek, fix this hack. */
	get_project_range (window);

	add_shell_listener (window);
	
	bonobo_ui_component_thaw (window->priv->component, NULL);	

	CORBA_exception_free (&ev);
	
	window_list = g_list_append (window_list, ret);

	gtk_widget_show_all (ret);
	
	return ret;
}

static void
project_window_add_control (ProjectWindow     *window,
			 Bonobo_Control  control_co,
			 GtkWidget      *widget,
			 GdkPixbuf      *pixbuf,
			 const gchar    *title)
{
	gtk_notebook_append_page (GTK_NOTEBOOK (window->priv->notebook),
				  widget, gtk_label_new (title));
	
	e_icon_bar_add_item (E_ICON_BAR (window->priv->shortcut_bar),
			     pixbuf, 
			     title, -1);

	window->priv->controls = g_list_append (window->priv->controls, control_co);
}

static gboolean
project_window_delete (ProjectWindow *window, gpointer user_data)
{
	if (project_window_really_exit (window)) {
		gtk_object_destroy (GTK_OBJECT (window));
		return FALSE;
	}

	return TRUE;
}

static void
project_window_exit (ProjectWindow *window)
{
	GList                          *list;

	list = g_list_copy (window_list);
	
	for (; list; list = list->next) {
		if (project_window_really_exit (list->data)) {
			gtk_object_destroy (GTK_OBJECT (list->data));
		}
	}
}

static gchar *
window_title_from_name (const gchar *str)
{
	return g_strdup_printf (_("%s - MrProject"), str);
}

static gchar *
get_project_name (ProjectWindow *window)
{
	GNOME_MrProject_Project project_co;
	CORBA_Environment ev;
	Bonobo_PropertyBag property_bag_co;
	Bonobo_Property prop_co;
	gchar *str, *title;

	CORBA_exception_init (&ev);
	
	project_co = shell_get_project (window->priv->shell);
	if (project_co == CORBA_OBJECT_NIL) {
		return NULL;
	}

	property_bag_co = Bonobo_Unknown_queryInterface (project_co, "IDL:Bonobo/PropertyBag:1.0", &ev);
	if (BONOBO_EX (&ev) || property_bag_co == CORBA_OBJECT_NIL) {
		g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);
		return NULL;
	}	
	
	prop_co = Bonobo_PropertyBag_getPropertyByName (property_bag_co, "Name", &ev);
	if (!BONOBO_EX (&ev)) {
		CORBA_any *any;
		
		any = Bonobo_Property_getValue (prop_co, &ev);
		str = BONOBO_ARG_GET_STRING (any);
		
		title = window_title_from_name (str);
		
		CORBA_free (any);
	} else {
		title = NULL;
	}
	
	bonobo_object_release_unref (prop_co, NULL);
	bonobo_object_release_unref (property_bag_co, NULL);
	CORBA_exception_free (&ev);

	return title;
}

#define IS_EVENT(event_name, str) (!strncmp (event_name, str, sizeof (str)-1))

static void
listener_callback (BonoboListener    *listener,
		   char              *event_name, 
		   CORBA_any         *any,
		   CORBA_Environment *ev,
		   gpointer           user_data)
{
	ProjectWindow *window;
	gchar      *subtype, *kind;

	window = PROJECT_WINDOW (user_data);

	subtype = bonobo_event_subtype (event_name);
	kind = bonobo_event_kind (event_name);

	if (IS_EVENT (event_name, "Bonobo/Property:")) {
		if (IS_EVENT (kind, "change")) {
			if (!strcmp (subtype, "Name")) {
				gchar *str, *title;

				str = BONOBO_ARG_GET_STRING (any);
				title = window_title_from_name (str);
				gtk_window_set_title (GTK_WINDOW (window), title);
				g_free (title);
			}
		}
	} else {
		g_print ("ProjectWindow: got unhandled event: %s\n", event_name);
	}

	g_free (subtype);
	g_free (kind);
}

static void
add_shell_listener (ProjectWindow *window)
{
	ProjectWindowPriv     *priv;
	CORBA_Environment      ev;
	GNOME_MrProject_Shell  shell_co;
	Bonobo_EventSource     event_source_co;
	Bonobo_Listener        listener_co;

	priv = window->priv;
	
	CORBA_exception_init (&ev);

	shell_co = BONOBO_OBJREF (priv->shell);
	event_source_co = Bonobo_Unknown_queryInterface (
		shell_co, "IDL:Bonobo/EventSource:1.0", &ev);
	if (BONOBO_EX (&ev) || !event_source_co) {
		CORBA_exception_free (&ev);
		return;
	}
	
	/* Listen to events from the shell. */
	priv->listener = bonobo_listener_new (listener_callback, window);
	listener_co = BONOBO_OBJREF (window->priv->listener);
	priv->listener_id = Bonobo_EventSource_addListenerWithMask (
		event_source_co,
		listener_co,
		"Bonobo/Property:change:Start,"	
		"Bonobo/Property:change:Finish,"
		"Bonobo/Property:change:Name",
		&ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
		return;
	}
	bonobo_object_release_unref (event_source_co, NULL);

	CORBA_exception_free (&ev);
}

static void
get_project_range (ProjectWindow *window)
{
	GNOME_MrProject_Project project_co;
	CORBA_Environment       ev;
	Bonobo_PropertyBag      pb_co;
	time_t                  start, finish;

	CORBA_exception_init (&ev);
	
	project_co = shell_get_project (window->priv->shell);
	if (!project_co) {
		return;
	}

	pb_co = Bonobo_Unknown_queryInterface (
		project_co, "IDL:Bonobo/PropertyBag:1.0", &ev);
	if (BONOBO_EX (&ev) || !pb_co) {
		g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);
		return;
	}	

	start = bonobo_property_bag_client_get_value_glong (
		pb_co, "Start", NULL);

	finish = bonobo_property_bag_client_get_value_glong (
		pb_co, "Finish", NULL);

	shell_set_range (window->priv->shell, start, finish);

	bonobo_object_release_unref (pb_co, NULL);
	CORBA_exception_free (&ev);
}

static void
confirm_exit_handle_yes (GtkWidget *widget, gboolean *exit_result)
{
	*exit_result = TRUE;
	gtk_main_quit ();
}

static void
confirm_exit_handle_no (GtkWidget *widget, gboolean *exit_result)
{
	*exit_result = FALSE;
	gtk_main_quit ();
}

static gboolean
project_window_really_exit (ProjectWindow *window)
{
	GNOME_MrProject_Project       project_co;
	CORBA_Environment             ev;
	GNOME_MrProject_ProjectState  state;
	GladeXML                     *glade;
	GtkWidget                    *dialog;
	GtkWidget                    *button_yes;
	GtkWidget                    *button_no;
	gboolean                      exit_result;

	project_co = shell_get_project (window->priv->shell);
	CORBA_exception_init (&ev);
	
	state = GNOME_MrProject_Project_getState (project_co, &ev);
	if (BONOBO_EX (&ev)) {
		g_log_exception (&ev, "");
		CORBA_exception_free (&ev);
		return TRUE;
	}

	if (state == GNOME_MrProject_PROJECT_STATE_SAVED ||
	    state == GNOME_MrProject_PROJECT_STATE_EMPTY) {
		return TRUE;
	}

	glade = glade_xml_new (MRPROJECT_DATADIR "confirm-exit-dialog.glade",
			       "confirm_exit_dialog");
	
	dialog = glade_xml_get_widget (glade, "confirm_exit_dialog");

	button_yes = glade_xml_get_widget (glade, "button_yes");
	button_no = glade_xml_get_widget (glade, "button_no");
	
	gtk_signal_connect (GTK_OBJECT (dialog),
			    "delete_event",
			    GTK_SIGNAL_FUNC (confirm_exit_handle_no),
			    &exit_result);
	
	gtk_signal_connect (GTK_OBJECT (button_yes),
			    "clicked",
			    GTK_SIGNAL_FUNC (confirm_exit_handle_yes),
			    &exit_result);

	gtk_signal_connect (GTK_OBJECT (button_no),
			    "clicked",
			    GTK_SIGNAL_FUNC (confirm_exit_handle_no),
			    &exit_result);
	
	gtk_main ();

	return exit_result;
}
