/*
 * Copyright (C) 2001  CodeFactory AB
 * Copyright (C) 2001  Richard Hult
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult <rhult@codefactory.se>
 */

#include <gtk/gtk.h>
#include "libmrproject/GNOME_MrProject.h"
#include "util/corba-utils.h"
#include "file-filter.h"

static void file_filter_init	   (FileFilter	    *file_filter);
static void file_filter_class_init (FileFilterClass *klass);

#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
static GtkObjectClass *parent_class;

BONOBO_X_TYPE_FUNC_FULL (FileFilter, 
			 GNOME_MrProject_FileFilter,
			 PARENT_TYPE,
			 file_filter);

static CORBA_boolean
impl_FileFilter_probe (PortableServer_Servant  servant,
		       const CORBA_char       *method,
		       const CORBA_char       *location,
		       CORBA_Environment      *ev)
{
	FileFilter      *filter;
	FileFilterClass *klass;

	filter = FILE_FILTER (bonobo_x_object (servant));
	klass = FILE_FILTER_CLASS (GTK_OBJECT (filter)->klass);
	
	if (klass->probe) {
		return klass->probe (filter, method, location, ev);
	}

	return FALSE;
}

static void
impl_FileFilter_load (PortableServer_Servant  servant,
		      const CORBA_char       *method,
		      const CORBA_char       *location,
		      const GM_Project        project,
		      CORBA_Environment      *ev)
{
	FileFilter       *filter;
	FileFilterClass  *klass;

	filter = FILE_FILTER (bonobo_x_object (servant));
	klass = FILE_FILTER_CLASS (GTK_OBJECT (filter)->klass);

	if (klass->load) {
		klass->load (filter, method, location, project, ev);
	}
}

static void
impl_FileFilter_loadSummary (PortableServer_Servant  servant,
			     const CORBA_char       *method,
			     const CORBA_char       *location,
			     GNOME_MrProject_FileSummary **summary,
			     CORBA_Environment      *ev)
{
	FileFilter       *filter;
	FileFilterClass  *klass;

	filter = FILE_FILTER (bonobo_x_object (servant));
	klass = FILE_FILTER_CLASS (GTK_OBJECT (filter)->klass);
	
	if (klass->load_summary) {
		klass->load_summary (filter, method, location, summary, ev);
	}
}

static void
impl_FileFilter_save (PortableServer_Servant  servant,
		      const CORBA_char       *method,
		      const CORBA_char       *location,
		      const GM_Project        project,
		      CORBA_Environment      *ev)
{
	FileFilter      *filter;
	FileFilterClass *klass;

	filter = FILE_FILTER (bonobo_x_object (servant));
	klass = FILE_FILTER_CLASS (GTK_OBJECT (filter)->klass);
	
	if (klass->save) {
		klass->save (filter, method,location, project, ev);
	}
}

static void
file_filter_class_init (FileFilterClass *klass)
{
	POA_GNOME_MrProject_FileFilter__epv *epv = &klass->epv;
	/*GtkObjectClass *object_class = (GtkObjectClass *) klass;*/

	parent_class = gtk_type_class (PARENT_TYPE);

	epv->probe       = impl_FileFilter_probe;
	epv->load        = impl_FileFilter_load;
	epv->loadSummary = impl_FileFilter_loadSummary;
	epv->save        = impl_FileFilter_save;
}

static void
file_filter_init (FileFilter *file_filter)
{
}


