/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2000-2001 CodeFactory AB
 * Copyright (C) 2000-2001 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2001      Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <signal.h>
#include <libgnomevfs/gnome-vfs.h>
#include <bonobo/bonobo-main.h>
#include <bonobo/bonobo-object-directory.h>
#include <liboaf/liboaf.h>
#include <orb/orbit.h>
#include "util/nautilus-debug.h"
#include "project-factory.h"

static ProjectFactory *project_factory;

static const struct poptOption  project_server_options [] = {
	{ NULL, '\0', 0, NULL, 0 }
};

int
main (int argc, char *argv [])
{
	CORBA_ORB          orb;
	
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	textdomain (PACKAGE);

	gnomelib_register_popt_table (oaf_popt_options, 
				      oaf_get_popt_table_name ());

	orb = oaf_init (argc, argv);

	gnome_init_with_popt_table ("mrproject-server", VERSION,
				    argc, argv, project_server_options, 
				    0, NULL);

	gnome_vfs_init ();

	if (bonobo_init (orb, CORBA_OBJECT_NIL,
			 CORBA_OBJECT_NIL) == FALSE) {
		g_error ("The project server component could not initialize Bonobo.");
	}

	if (g_getenv ("MRPROJECT_DEBUG") != NULL) {
		nautilus_make_warnings_and_criticals_stop_in_debugger
			(G_LOG_DOMAIN, g_log_domain_glib,
			 "Bonobo",
			 "GnomeUI",
			 "GnomeVFS",
			 "GnomeVFS-CORBA",
			 "GnomeVFS-pthread",
			 "Gtk",
			 "MrProject",
			 "ORBit",
			 "Util",
			 "Backend",
			 NULL);
	}
		
	project_factory = project_factory_new ();
	
/*
	signal (SIGSEGV, SIG_DFL);
	signal (SIGBUS, SIG_DFL);
*/

	g_print ("MrProject server started.\n");
	
	bonobo_main ();
	
	bonobo_object_unref (BONOBO_OBJECT (project_factory));
	project_factory = NULL;

	return 0;
}
