/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2001 CodeFactory AB
 * Copyright (C) 2001 Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Mikael Hallendal <micke@codefactory.se>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-util.h>
#include "string-utils.h"

gboolean 
split_uri                 (const gchar    *uri, 
                           gchar         **method, 
                           gchar         **location)
{
        gchar **s, *s1, *s2;

        s = g_strsplit (uri, ":", 2);

        if (s[0] == NULL || s[1] == NULL) {
                g_strfreev (s);
                return FALSE;
        }

        s1 = g_strdup (s[0]);
        s2 = g_strdup (s[1]);

        *method = s1;
        *location = s2;

        g_strfreev (s);

        return TRUE;
}

gboolean 
create_uri                (gchar         **uri,
                           const gchar    *method,
                           const gchar    *location)
{
        *uri = g_strdup_printf ("%s://%s", 
                                method,
                                location);

        return TRUE;
}

gboolean 
create_uri_from_filename  (gchar         **uri,
			   const gchar    *filename)
{
	gchar         *fixed_filename;
	gboolean       ret;
	
	if (!g_path_is_absolute (filename)) {
		fixed_filename = g_concat_dir_and_file (g_get_current_dir (),
							filename);
	} else {
		fixed_filename = g_strdup (filename);
	}

	ret = create_uri (uri, "file", fixed_filename);
	
	g_free (fixed_filename);
	
	return ret;
}

